<?php

namespace Drupal\social_auth_zalo\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\social_api\Plugin\NetworkManager;
use Drupal\social_auth\SocialAuthUserManager;
use Drupal\social_auth_zalo\ZaloAuthManager;
use Drupal\social_auth_zalo\ZaloAuthPersistentDataHandler;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Controller for Social Auth Zalo.
 */
class ZaloAuthController extends ControllerBase {

  /**
   * The network plugin manager.
   *
   * @var \Drupal\social_api\Plugin\NetworkManager
   */
  private $networkManager;

  /**
   * The user manager.
   *
   * @var \Drupal\social_auth\SocialAuthUserManager
   */
  private $userManager;

  /**
   * Used to access GET parameters.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  private $request;

  /**
   * The Zalo Persistent Data Handler.
   *
   * @var \Drupal\social_auth_zalo\ZaloAuthPersistentDataHandler
   */
  private $persistentDataHandler;

  /**
   * The Zalo authentication manager.
   *
   * @var \Drupal\social_auth_zalo\ZaloAuthManager
   */
  private $zaloManager;

  /**
   * ZaloAuthController constructor.
   *
   * @param \Drupal\social_api\Plugin\NetworkManager $network_manager
   *   Used to get an instance of social_auth_zalo network plugin.
   * @param \Drupal\social_auth\SocialAuthUserManager $user_manager
   *   Manages user login/registration.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   *   Used to access GET parameters.
   * @param \Drupal\social_auth_zalo\ZaloAuthPersistentDataHandler $persistent_data_handler
   *   ZaloAuthPersistentDataHandler object.
   * @param \Drupal\social_auth_zalo\ZaloAuthManager $zalo_manager
   *   ZaloAuthManager object.
   */
  public function __construct(NetworkManager $network_manager, SocialAuthUserManager $user_manager, RequestStack $request, ZaloAuthPersistentDataHandler $persistent_data_handler, ZaloAuthManager $zalo_manager) {
    $this->networkManager = $network_manager;
    $this->userManager = $user_manager;
    $this->request = $request;
    $this->persistentDataHandler = $persistent_data_handler;

    // Sets the plugin id.
    $this->userManager->setPluginId('social_auth_zalo');

    $this->zaloManager = $zalo_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.network.manager'),
      $container->get('social_auth.user_manager'),
      $container->get('request_stack'),
      $container->get('social_auth_zalo.persistent_data_handler'),
      $container->get('social_auth_zalo.manager')
    );
  }

  /**
   * Redirect to Zalo login url.
   */
  public function redirectToZaloLogin() {
    /* @var \Zalo\Zalo|false $zalo */
    $zalo = $this->networkManager->createInstance('social_auth_zalo')->getSdk();

    // If zalo client could not be obtained.
    if (!$zalo) {
      drupal_set_message($this->t('Social Auth Zalo not configured properly. Contact site administrator.'), 'error');
      return $this->redirect('user.login');
    }

    $this->zaloManager->setClient($zalo);
    $zalo_login_url = $this->zaloManager->getZaloLoginUrl();

    return new TrustedRedirectResponse($zalo_login_url);
  }

  /**
   * Response for path 'user/login/zalo/callback'.
   */
  public function returnFromZalo() {
    // Get return code from url parameters.
    $code = $this->request->getCurrentRequest()->get('code');

    if (empty($code)) {
      drupal_set_message($this->t('You could not be authenticated.'), 'error');
      return $this->redirect('user.login');
    }

    /* @var \Zalo\Zalo|false $zalo */
    $zalo = $this->networkManager->createInstance('social_auth_zalo')->getSdk();

    // If zalo client could not be obtained.
    if (!$zalo) {
      drupal_set_message($this->t('Social Auth Zalo not configured properly. Contact site administrator.'), 'error');
      return $this->redirect('user.login');
    }

    $this->zaloManager->setClient($zalo)->authenticate();

    if (!$zalo_profile = $this->zaloManager->getUserInfo()) {
      drupal_set_message($this->t('Zalo login failed, could not load Zalo profile. Contact site administrator.'), 'error');
      return $this->redirect('user.login');
    }
    // Gets user's id from the Zalo profile.
    if (!$id = $this->zaloManager->getId($zalo_profile)) {
      drupal_set_message($this->t('Zalo login failed. This site requires permission to get your Id.'), 'error');
      return $this->redirect('user.login');
    }
    // Gets user's name from the Zalo profile.
    if (!$name = $this->zaloManager->getName($zalo_profile)) {
      drupal_set_message($this->t('Zalo login failed. This site requires permission to get your Name.'), 'error');
      return $this->redirect('user.login');
    }

    // Zalo API dont have user email.
    // Just Id and Name return from their API.
    // Ex: ["id": "2342343242342", "name": "Lap Pham"].
    // So temporary use the id to authenticate.
    // Will find another solution from other expert developer.
    return $this->userManager->authenticateUser($id, $name, $id);
  }

}
